/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// NotesSettings.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "NotesSettings.h"
#include "MainSyncFrm.h"
#include "ClientUtil.h"

#include "winmaincpp.h"
#include "utils.h"
#include "comutil.h"
#include "OutlookPlugin.h"

#include <string>

using namespace std;
// CNotesSettings

static wstring outlookFolder;
static CNotesSettings* wndNotes;
static HANDLE handleThread;

IMPLEMENT_DYNCREATE(CNotesSettings, CDialog)

CNotesSettings::CNotesSettings()
	: CDialog(CNotesSettings::IDD)
{

}

CNotesSettings::~CNotesSettings()
{
    // clean stuff used in the select Outlook folder thread
    wndNotes = NULL;
    CloseHandle(handleThread);
}

void CNotesSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_NOTES_COMBO_SYNCTYPE, lstSyncType);
    DDX_Control(pDX, IDC_NOTES_EDIT_FOLDER, editFolder);
    DDX_Control(pDX, IDC_NOTES_CHECK_INCLUDE, checkInclude);
    DDX_Control(pDX, IDC_NOTES_BUT_SELECT, butSelectFolder);
    DDX_Control(pDX, IDC_NOTES_EDIT_REMOTE, editRemote);
    DDX_Control(pDX, IDC_NOTES_GROUP_DIRECTION, groupDirection);
    DDX_Control(pDX, IDC_NOTES_GROUP_FOLDER, groupFolder);
    DDX_Control(pDX, IDC_NOTES_GROUP_ADVANCED, groupAdvanced);
}

BEGIN_MESSAGE_MAP(CNotesSettings, CDialog)
    ON_BN_CLICKED(IDC_NOTES_OK, &CNotesSettings::OnBnClickedNotesOk)
    ON_BN_CLICKED(IDC_NOTES_CANCEL, &CNotesSettings::OnBnClickedNotesCancel)
    ON_BN_CLICKED(IDC_NOTES_BUT_SELECT, &CNotesSettings::OnBnClickedNotesButSelect)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

// CNotesSettings diagnostics

#ifdef _DEBUG
void CNotesSettings::AssertValid() const
{
	CDialog::AssertValid();
}

#ifndef _WIN32_WCE
void CNotesSettings::Dump(CDumpContext& dc) const
{
	CDialog::Dump(dc);
}
#endif
#endif //_DEBUG


// CNotesSettings message handlers

BOOL CNotesSettings::OnInitDialog(){
    CString s1;
    s1.LoadString(IDS_NOTES_DETAILS); SetWindowText(s1);
    CDialog::OnInitDialog();
    
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(NOTE_);

    editFolder.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editRemote.SetLimitText(EDIT_TEXT_MAXLENGTH);
    
    // load string resources
    s1.LoadString(IDS_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE2); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE3); lstSyncType.AddString(s1);
    
    s1.LoadString(IDS_SYNC_DIRECTION); SetDlgItemText(IDC_NOTES_GROUP_DIRECTION, s1);
    s1.LoadString(IDS_NOTES_FOLDER); SetDlgItemText(IDC_NOTES_GROUP_FOLDER, s1);
    s1.LoadString(IDS_CURRENT); SetDlgItemText(IDC_NOTES_STATIC_FOLDER, s1);
    s1.LoadString(IDS_INCLUDE_SUBFOLDERS); SetDlgItemText(IDC_NOTES_CHECK_INCLUDE, s1);
    s1.LoadString(IDS_SELECT_FOLDER); SetDlgItemText(IDC_NOTES_BUT_SELECT, s1);
    s1.LoadString(IDS_REMOTE_NAME); SetDlgItemText(IDC_NOTES_STATIC_REMOTE, s1);
    s1.LoadString(IDS_DATA_FORMAT); SetDlgItemText(IDC_NOTES_STATIC_DATAFORMAT, s1);
    s1.LoadString(IDS_USE_SIF); SetDlgItemText(IDC_NOTES_STATIC_SIF, s1);
    s1.LoadString(IDS_ADVANCED); SetDlgItemText(IDC_NOTES_GROUP_ADVANCED, s1);

    s1.LoadString(IDS_OK); SetDlgItemText(IDC_NOTES_OK, s1);
    s1.LoadString(IDS_CANCEL); SetDlgItemText(IDC_NOTES_CANCEL, s1);

    //-- load settings from Registry
    lstSyncType.SetCurSel(getSyncTypeIndex(ssconf->getSync()));
    
    // get folder path
    s1 = ssconf->getFolderPath(); 
    if(s1 == ""){
        s1 = getDefaultFolderPath(NOTE).data();
    };
    SetDlgItemText(IDC_NOTES_EDIT_FOLDER, s1);

    if(ssconf->getUseSubfolders())
        checkInclude.SetCheck(BST_CHECKED);

    s1 =  ssconf->getURI();
    SetDlgItemText(IDC_NOTES_EDIT_REMOTE, s1);

    wndNotes = this;

    // disable windows xp theme, otherwise any color setting for groupbox
    // will be overriden by the theme settings
    if(((COutlookPluginApp*)AfxGetApp())->hLib){
        PFNSETWINDOWTHEME pfnSetWindowTheme =
            (PFNSETWINDOWTHEME)GetProcAddress(((COutlookPluginApp*)AfxGetApp())->hLib, "SetWindowTheme");
        pfnSetWindowTheme (groupDirection.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupFolder.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupAdvanced.m_hWnd,L" ",L" ");
    };
    
    // Accessing Outlook, could be no more in foreground
    SetForegroundWindow();

    return FALSE;
}


void CNotesSettings::OnBnClickedNotesOk()
{
    // OK Button
    if(saveSettings(false)){
        CDialog::OnOK();
    }
}

void CNotesSettings::OnBnClickedNotesCancel()
{
    // CANCEL button
    getConfig()->read();
    CDialog::OnCancel();
}

bool CNotesSettings::saveSettings(bool saveToDisk)
{
    CString remoteName, outlookFolder, syncType;
    CString s1;
    _bstr_t bst;
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(NOTE_);

    GetDlgItemText(IDC_NOTES_EDIT_REMOTE, remoteName);
    GetDlgItemText(IDC_NOTES_EDIT_FOLDER, outlookFolder);

    // change values
        if(remoteName == ""){
            s1.LoadString(IDS_ERROR_SET_REMOTE_NAME);
            MessageBox(s1);
            return false;
        };
        // sync source enabled
        ssconf->setSync(getSyncTypeName(lstSyncType.GetCurSel()));

        bst.Assign(outlookFolder.AllocSysString());
        ssconf->setFolderPath(bst); bst.Detach();

        if(checkInclude.GetCheck() == BST_CHECKED)
            ssconf->setUseSubfolders(true);
        else
            ssconf->setUseSubfolders(false);

        bst.Assign(remoteName.AllocSysString());
        ssconf->setURI(bst); bst.Detach();

    if(saveToDisk)
        ((OutlookConfig*)getConfig())->save();

    return true;
}

int pickFolderNotes(){
    CString s1;
    outlookFolder = pickOutlookFolder(NOTE);
    if ( (outlookFolder != L"") && (wndNotes != NULL) ){
        s1 = outlookFolder.data();
        wndNotes->SetDlgItemText(IDC_NOTES_EDIT_FOLDER, s1);
    }
    wndNotes->SetForegroundWindow();
    return 0;
}

void CNotesSettings::OnBnClickedNotesButSelect()
{
    outlookFolder = L"";
    handleThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)pickFolderNotes, NULL, 0, NULL);
}

HBRUSH CNotesSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomGroupbox)){
        // do not check the id
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(COLOR_GROUPBOX_BLACK);
    };

    return hbr;
}
