/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "base/util/utils.h"
#include "winmaincpp.h"
#include "DefaultWinConfigFactory.h"
#include <string>

using namespace std;


DefaultWinConfigFactory::DefaultWinConfigFactory() : DefaultConfigFactory() {
}

DefaultWinConfigFactory::~DefaultWinConfigFactory() {
}


AccessConfig* DefaultWinConfigFactory::getAccessConfig() {

    AccessConfig* ac = new AccessConfig();

    ac->setUsername             ("guest");                              // This should be replaced, set by installer.
    ac->setPassword             ("guest");                              // This should be replaced, set by installer.
    ac->setSyncURL              ("http://localhost:8080/funambol/ds");  // This should be replaced, set by installer.
    ac->setFirstTimeSyncMode    (SYNC_NONE);
    ac->setUseProxy             (FALSE);
    ac->setProxyHost            ("");
    ac->setProxyPort            (8080);
    ac->setProxyUsername        ("");
    ac->setProxyPassword        ("");
    ac->setBeginSync            (0);
    ac->setEndSync              (0);
    ac->setServerAuthRequired   (FALSE);
    ac->setClientAuthType       ("syncml:auth-basic");
    ac->setServerAuthType       ("syncml:auth-basic");
    ac->setServerPWD            ("funambol");
    ac->setServerID             ("funambol");
    ac->setServerNonce          ("");
    ac->setClientNonce          ("");
    ac->setMaxMsgSize           (250000);
    ac->setMaxModPerMsg         (100);
    ac->setReadBufferSize       (0);
    ac->setUserAgent            (PROGRAM_NAME);                         // This should be replaced, adding sw version.
    ac->setCheckConn            (FALSE);
    ac->setResponseTimeout      (0);

    return ac;
}



DeviceConfig* DefaultWinConfigFactory::getDeviceConfig() {

    DeviceConfig* dc = new DeviceConfig();

    dc->setVerDTD               ("1.1");
    dc->setMan                  ("Funambol");
    dc->setMod                  ("Funambol Outlook Plug-in");
    dc->setOem                  ("");
    dc->setFwv                  ("");
    dc->setSwv                  ("");                                   // This MUST be replaced, set by installer.
    dc->setHwv                  ("");
    dc->setDevID                ("fol-default");                        // This should be generated unique with config.setUniqueDevID()
    dc->setDevType              ("workstation");
    dc->setDsV                  ("");
    dc->setUtc                  (TRUE);
    dc->setLoSupport            (TRUE);
    dc->setNocSupport           (TRUE);
    dc->setLogLevel             (LOG_LEVEL_INFO);
    dc->setMaxObjSize           (0);
    dc->setDevInfHash           ("");

    return dc;
}



SyncSourceConfig* DefaultWinConfigFactory::getSyncSourceConfig(const wstring& wname) {

    SyncSourceConfig* sc = new SyncSourceConfig();
    char* name = toMultibyte(wname.c_str());

    sc->setName                 (name);
    sc->setSyncModes            ("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client");
    sc->setEncoding             ("b64");
    sc->setLast                 (0);
    sc->setSync                 ("two-way");
    sc->setVersion              ("");
    sc->setEncryption           ("");

    if (wname == CONTACT){
        sc->setURI              ("scard");
        sc->setType             ("text/x-s4j-sifc");
        sc->setSupportedTypes   ("text/x-s4j-sifc:,text/x-vcard:2.1");
    }
    else if (wname == APPOINTMENT){
        sc->setURI              ("scal");
        sc->setType             ("text/x-s4j-sife");
        sc->setSupportedTypes   ("text/x-s4j-sife:,text/x-vcalendar:1.0");
    }
    else if (wname == TASK){
        sc->setURI              ("stask");
        sc->setType             ("text/x-s4j-sift");
        sc->setSupportedTypes   ("text/x-s4j-sift");
    }
    else if (wname == NOTE){
        sc->setURI              ("snote");
        sc->setType             ("text/x-s4j-sifn");
        sc->setSupportedTypes   ("text/x-s4j-sifn");
    }

    if (name) delete [] name;
    return sc;
}




WindowsSyncSourceConfig* DefaultWinConfigFactory::getWinSyncSourceConfig(const wstring& wname, SyncSourceConfig* sc) {

    WindowsSyncSourceConfig* wsc = new WindowsSyncSourceConfig(sc);

    wsc->setUseSubfolders       (true);
    wsc->setFolderPath          ("");
    wsc->setEndTimestamp        (0);

    return wsc;
}

