/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef INCL_OUTLOOKCONFIG
#define INCL_OUTLOOKCONFIG

#include "base/Log.h"
#include "spds/SyncReport.h"
#include "Client/DMTClientConfig.h"
#include "WindowsSyncSourceConfig.h"


// Client specific properties 
// (see spdm/costants.h for common props)
#define PROPERTY_USE_SUBFOLDERS                 "useSubfolders"
#define PROPERTY_FOLDER_PATH                    "folderPath"
#define PROPERTY_SCHEDULED_SYNC                 "isScheduled"

// This is stored in HKLM during install
#define PROPERTY_SP                             "portal"
#define PROPERTY_INSTALLDIR                     "installDir"

// Path in DMTree
#define APPLICATION_URI                         "Funambol/OutlookClient"

// Default portal settings
#define PORTAL_DEFAULT_SYNCURL                  "http://funambol.net/sync"
#define PORTAL_DEFAULT_USERNAME                 ""
#define PORTAL_DEFAULT_PASSWORD                 ""



/******************************************************************************
* Class OutlookConfig:
* 
* Rapresents the configuration of Outlook client.
* This class is derivated from DMTClientConfig, which is derivated from
* SyncMangerConfig.
*******************************************************************************
*/
class OutlookConfig : public DMTClientConfig {

private:

    // pointer to OutlookConfig instance
    static OutlookConfig* pinstance;

    char* workingDir;                           // The path of current working directory
    char* logDir;                               // The path of current log directory (under app data)
    bool  fullSync;                             // true if we are running a full sync (slow/refresh)
    //bool  scheduledSync;                        // true if we are running a scheduled sync (set during initialize())
    bool  abortSync;                            // set to true when we want to (soft) abort the current sync


    // Array of specific SSConfig.
    WindowsSyncSourceConfig** winSourceConfigs;

    // Counter for winSourceConfigs array.
    // (internal use, should be equal to 'sourceConfigsCount' member)
    unsigned int winSourceConfigsCount;


    void readWinSourceConfig(unsigned int i);
    void saveWinSourceConfig(unsigned int i);


    // Returns the value of the given property, from rootKey tree (read only).
    char* readPropertyValue(const char* context, const char* propertyName, HKEY rootKey);

protected:

    // Constructor
    OutlookConfig();


public:
    
    // Method to get the sole instance of OutlookConfig
    static OutlookConfig* getInstance();

    // Returns true if static instance is not NULL.
    static bool isInstantiated();

    // Destructor
    ~OutlookConfig();


    // Override read/save methods of DMT (use specific winSourceConfig)
    BOOL read();
    BOOL save();
    BOOL save(SyncReport* report);


    // Read all sources timestamps from win registry.
    void readSourcesTimestamps();


    // Replace getSyncSourceConfig() of DMT (return specific winSourceConfig)
    _declspec(dllexport) WindowsSyncSourceConfig* getSyncSourceConfig(const char* name);
    WindowsSyncSourceConfig* getSyncSourceConfig(unsigned int i);

    // Replace setSyncSourceConfig() of SyncManagerConfig (set specific winSourceConfig)
    BOOL setSyncSourceConfig(WindowsSyncSourceConfig& wsc);
    BOOL addSyncSourceConfig(WindowsSyncSourceConfig& wsc);


    // get/set of internal members
    void setWorkingDir   (const char* v);
    void setLogDir       (const char* v);
    void setFullSync     (const  bool v);
    void setScheduledSync(const  bool v);
    void setAbortSync    (const  bool v);

    const char* getWorkingDir()    const;
    _declspec(dllexport) const char* getLogDir()        const;
    const bool  getFullSync()      const;
    const bool  getScheduledSync() const;
    const bool  getAbortSync()     const;


    // Creates a default configuration.
    void createDefaultConfig();

    // Update config with values from HKLM (set by installer).
    void upgradeConfig();

    // Returns the current software version, read it from HKLM registry.
    char* readCurrentSwv();

    //Creates and set a unique 'devID' property for current configuration.
    int setUniqueDevID();

    // Check if it's a normal/portal build (from HKLM keys).
    bool checkPortalBuild();


    // Save only "beginSync" property to win registry.
    void saveBeginSync();
    // Save only "sync" properties of each source, to win registry.
    void saveSyncModes();
};

#endif